/*
 * A miniture shell program that is used for illustration
 * in CSA 381.  This particular version was programmed
 * by Alton Sanders but draws heavily on examples by a host
 * of authors. Among them are Andrew Tanenbaum, Gary Nutt, Michael
 * Johnson, Erik Troan, and William Stallings.  I no longer know
 * exacly what came from whom, but I am confident that any bugs
 * that are here belong to me.
 *
 */


#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "minishell.h"

/* Function prototypes */
int parseCommand(char *, struct command_t *);
char* lookupPath(char *[], char *[]);

/********* Local Functions  ****************************/

void readCommand(char *buffer){
  /* reads in a command from the command line
   * no globbing, classy editing, or any other sexy stuff
   *
   */
  fgets(buffer, MAX_LINE_LEN,stdin);
}

void printPrompt(){
  /* Collects the current working directory from the
   * environment variable PWD (if PWD is currently set)
   * concatenated with a 'base prompt' identifying
   * the shell.
   * If PWD is not currently set, it simply gives the
   * base prompt.
   *
   * It would actually be better (certainly faster) to
   * use static string space rather than dynamically
   * allocating memory, but this provides an easy way
   * of demonstrating malloc() and free().
   */
  char* promptString;
  char  basePrompt[] = ":minishell>";
  char* pwd;

  pwd = (char *) getenv("PWD");

  if (pwd == NULL) promptString = basePrompt;
  else {
    promptString = (char *) malloc(strlen(pwd) + strlen(basePrompt) + 1);
    strcpy(promptString, pwd);
    strcat(promptString, basePrompt);
  }

  printf("%s", promptString);
  free(promptString);
}

int is_exit(char *cmd_name){
  /* compare strings ignoring case
   * zero (false) means strings are equal
   */
  if (strcasecmp(cmd_name, "quit") == 0) return 1;
  else if (strcasecmp(cmd_name, "exit") == 0) return 1;
	   else return 0;
}

int is_special(struct command_t command){
  /* compare strings ignoring case
   * zero (false) means strings are equal
   */	
	char* pathString;
	if (strcasecmp(command.name, "cd") == 0){
	//	printf("chdir(%s) returned: %i PWD: %s\n", command.argv[1], chdir(command.argv[1]), (char *) getenv("PWD"));
		if(chdir(command.argv[1]) == 0){
			pathString = (char *) malloc(MAX_PATH_LEN);
			getcwd(pathString, MAX_PATH_LEN);
			setenv("PWD", pathString, 1);
			free(pathString);
		}else 
			printf("minishell: cd: %s: No such file or directory\n", command.argv[1]);

		return 1;
	}else return 0;
}

/**********************  Main Program **********************************/

int main(int argc, char *argv[]){
  int i;
  int pid, numChildren;
  int status;
  FILE *fid;
  char cmdLine[MAX_LINE_LEN];
  struct command_t command;
  char *pathv[MAX_PATHS];

	char savedLine[MAX_LINE_LEN];
  /* Determine directory Paths.  Note that we do not account for the
   * paths changing due to any of our commands -- a definite weakness
   * in our shell.
   */
  printf("Entering minishell\n");
  fflush(stdout);

  parsePath(pathv);

  /* main read/execute loop */
  while (TRUE) {
    printPrompt();
    readCommand(cmdLine);
	strcpy(savedLine, cmdLine);
    parseCommand(cmdLine, &command);

    /* check for exit command */
    if (is_exit(command.name)) break;

    if (is_special(command)) continue;

    /* Modify the command name to reflect the full path
     */
    command.name = lookupPath(command.argv,pathv);

    /* Make sure the last item in the arg vector is followed
     * by NULL
     */
    command.argv[command.argc] = NULL;

    /* Create a child process to execute the command */
    if ((pid = fork()) == 0) {
      /* Child executing command */
     // execv(command.name, command.argv);
	char *cmd[] = { "sh", "-c", savedLine, (char *)0 };
	execv("/bin/sh", cmd);
      exit(0); /* This should never be executed */
    }
    /* Parent will wait until the command is finished */
    waitpid(pid,&status,0);
  }
  printf("Exiting Minishell\n");
  return 0;
}
