/*
 * Code adapted from from that in the book Operating Systems
 * by Gary Nutt.  The code here is essentially the same as
 * Nutt's except that a memory leak has been removed.
 *
 */

#include <string.h>

#include "minishell.h"


/* Determine command name and construct the parameter list.
 * This function will build argv[] and the set the argc value.
 *
 * argc is the number of "tokens" or words on the command line
 * argv[] is an array of strings (pointers to char *0).  The last
 * element of argv[] must be NULL.
 *
 * As we scan the command line from the left, the first token
 * goes in argv[0], the second in argv[1], and so on.  Each
 * time we add a token to argv[], we increment argc.
 */

int parseCommand(char *cLine, struct command_t *cmd){
  int argc;
  char **clPtr;
  /* Initialization */
  clPtr = &cLine;     /* cLine is the command line */
  argc = 0;
  cmd->argv[argc] = (char *) malloc(MAX_ARG_LEN);
  /* Fill argv[] */
  while ((cmd->argv[argc] = strsep(clPtr, WHITESPACE)) != NULL) {
    cmd->argv[++argc] = (char *) malloc(MAX_ARG_LEN);
  }

  /* Set the command name and argc */
  cmd->argc = argc-1;
  cmd->name = (char *) malloc(sizeof(cmd->argv[0]));
  strcpy(cmd->name, cmd->argv[0]);
  return 1;
  }
